/* -*- C++ -*-
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * 4758_sym_crypto.h
 * alex iliev, apr 2003
 * declarations for symmetric crypto provider classes using 4758 crypto
 * interface
 */

#include <string>

#include <leeds/crypto/crypto.h>

#include <common/exceptions.h>
#include <common/utils.h>
#include <common/sym_crypto.h>


#ifndef _4758_SYM_CRYPTO_H
#define _4758_SYM_CRYPTO_H




class SymCrypt4758 : public SymCryptProvider {

public:

    SymCrypt4758 () throw (crypto_exception);
    
    virtual void symcrypto_op (const ByteBuffer& input,
			       const ByteBuffer& key,
			       const ByteBuffer& iv,
			       ByteBuffer & out,
			       SymCryptProvider::OpType optype)
	throw (crypto_exception);

    virtual ~SymCrypt4758();

private:

    des_req_type _req;
    int          _fd;
};



class SHA1HMAC_4758 : public MacProvider {

public:

    SHA1HMAC_4758 () throw (crypto_exception);

    
    virtual void genmac (const ByteBuffer& text, const ByteBuffer& key,
			 ByteBuffer & out)
	throw (crypto_exception);

    ~SHA1HMAC_4758();


private:
    
    hmacsha1_req_type _req;
    int               _fd;	// the device file fd
};





#endif // _4758_SYM_CRYPTO_H
