/* -*- c++ -*-
 * io.h
 * alex iliev, may 2003
*/

#ifndef _CARD_IO_H
#define _CARD_IO_H

#include <string>

#include <common/comm_types.h>
#include <common/utils.h>
#include <common/exceptions.h>


// need to support read, write, and a "base dir"


#if 0
void host_write_obj (const std::string& basedir,
		     const object_id& id,
		     const ByteBuffer& obj)
    throw (host_exception);

ByteBuffer host_read_obj (const std::string& basedir,
			  const object_id& id)
    throw (host_exception);

#endif // 0


//
// class to provide reading and writing of objects on the host
// 
// FIXME: the naming of objects via object_id is a bit arbitrary, should make it
// a bit more generic:
// - a class hierarchy
// - a string
// - later...
// 
// TODO: should be able to add encryption later on, so it is done below this IO
// interface, as opposed to above it:
// - will need to provide crypto objects
// - and perhaps change them
// 
//

//
// an exception class
//
class hostio_exception : public better_exception {

public:
    hostio_exception (const std::string& msg) : better_exception (msg) {}

};



class HostIO {

public:

    HostIO (const std::string & basedir);
    
    void write (const object_id& id,
		const ByteBuffer& obj)
	throw (hostio_exception);

    ByteBuffer read (const object_id& id)
	throw (hostio_exception);

    //
    // and the same things but using string naming
    //
    void write (const std::string& name,
		const ByteBuffer& obj)
	throw (hostio_exception);

    ByteBuffer read (const std::string& name)
	throw (hostio_exception);



private:
    std::string _basedir;

};





#endif // _CARD_IO_H
