/* -*- c++ -*-
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/* -*-c++-*-
 * shuffle.h
 * alex iliev, jan 2003
 * the database shuffling code
 */


#ifndef _SHUFFLE_H
#define _SHUFFLE_H


#include <vector>

#include <stdlib.h>

#include <common/utils.h>
#include <common/exceptions.h>
#include <common/sym_crypto.h>

#include "io.h"

#include "pirsession.h"		// for Bucket
#include "permute.h"


class Shuffler {

public:

    Shuffler (const std::string& crypt_dirname,
	      SymCryptProvider & symop,
	      MacProvider & macop,
	      const ByteBuffer & key,
	      const ByteBuffer & mackey,
	      size_t N, size_t bucket_size);
    
    void generate_shuffle ();

    // this must be symmetrical with the shuffle load procedure during
    // retrieval!
    // note that 'filename' goes inside the top dir of the current shuffled DB
    void write_shuffle (const std::string& filename)
	throw (std::ios::failure, hostio_exception);

    void shuffle ()
	throw (host_exception, crypto_exception);


private:

    ByteBuffer _key, _mackey;

    SymWrapper _sym_wrapper;
//     SymDencrypter _enc;
//     MacExpert _mac;

    HostIO _crypt_io, _clear_io;
    
    perm_vector V;
    size_t N;
    size_t _bucket_size;


    //
    // class to do a switch in the Benes network
    //
    class Switcher {

    public:
	
	Switcher (Shuffler & master) : master(master) {}

	void operator () (index_t a, index_t b, bool should_swap)
	    throw (host_exception, crypto_exception);

    private:
	Shuffler & master;
	
    };

    friend class Switcher;
    
    //
    // helpers
    //

    // make an encrypted database
    void make_encrypted_db () throw (host_exception, crypto_exception);
    

    // fetch a cleartext bucket into o_records
    void fetch_clear_bucket (index_t n, Bucket & o_records)
	throw (host_exception);

    // encrypt and write a bucket
    void write_bucket (index_t n, const Bucket & bucket)
	throw (host_exception, crypto_exception);


    // fetch an encrypted bucket and decrypt it into blobs
    void fetch_blobs (index_t n, std::list<ByteBuffer> & o_records)
	throw (host_exception, crypto_exception);
    
    // encrypt and write list of blobs representing a bucket
    void write_blobs (index_t n, const std::list<ByteBuffer> & bucket)
	throw (host_exception, crypto_exception);
    

};


// set V_[V[i]] = i for all indices i
void transpose_vector (std::vector<index_t> & V_,
		       const std::vector<index_t>& V);


#endif // _SHUFFLE_H
