/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * pirsearch.cc
 * alex iliev, jan 2003
 * the main host-side program for querying the PIR server
 */

#include <iostream>

#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>		// for getopt()


#ifdef _TIMING
#include <sys/time.h>
#include <time.h>
#endif

#include <common/utils.h>
#include <common/consts.h>
#include <common/socket-class.h>
#include <common/exceptions.h>

#include <common/countptr.hpp>



using namespace std;


static std::string query_card (const std::string& name,
			       DatagramSocket & sock,
			       const SocketAddress & card_addr)
    throw (comm_exception)
{
    counted_ptr<SocketAddress> peer_addr;
    
    sock.send (ByteBuffer(name), card_addr);
    ByteBuffer reply = sock.recv (peer_addr);

    return std::string (reply.cdata(), reply.len());
}



static void
usage(char *prog)
{
    fprintf(stderr, "Usage: %s cardno [-p port]\n",prog);
}




int main (int argc, char *argv[]) {

    unsigned short cardno = 1,		// the first card
	serverport = PIR_SEARCH_PORT;


    if ((argc < 2) ||(argc > 5)){
	usage(argv[0]);
	exit(0);
    }

    cardno = atoi (argv[1]);
    
    int i;
    while((i = getopt(argc, argv, "p:")) != EOF) {
	switch(i) {
	case('p'):
	    serverport = atoi(optarg); break;

	default:
	    usage(argv[0]);
	    exit(EXIT_FAILURE); break;
	} /* switch options */
    }


    std::string buf, answer;
    if (!getline (cin, buf)) {
	cerr << "Error reading request" << endl;
	exit(EXIT_FAILURE);
    }

    try {
	// set up the socket to the card
	SCCDatagramSocket sock;
	SCCSocketAddress card_addr (cardno, serverport);
	
	answer = query_card (buf, sock, card_addr);
    }
    catch (const comm_exception & ex) {
	cerr << "Error sending query to card: " << ex.what() << endl;
	exit (EXIT_FAILURE);
    }
    
    cout << answer << endl;
}
